----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    18:14:20 03/21/2011 
-- Design Name: 
-- Module Name:    DDR2_Write_VHDL - Verhalten 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

---- Uncomment the following library declaration if instantiating
---- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

	--------------------------------------------
	-- Beschreibung :
	--
	-- Die State-Machine schreibt
	-- einen 32Bit Wert ins RAM
	--
	-- die Adresse und Daten werden
	-- von der bergeordneten CONTROL-Unit
	-- gehandelt
	--
	-- hier werden nur (nach Write-Enable)
	-- die Commandos an das RAM
	-- gehandelt
	-- 
	-- solange die Write-Funktion luft,
	-- ist BUSY=1
	--------------------------------------------

entity DDR2_Write_VHDL is

	--------------------------------------------
	-- Port Deklerationen
	--------------------------------------------
	port (
		reset_in : in std_logic;
		clk_in : in std_logic;
		w_command_register : out std_logic_vector(2 downto 0); -- geht ans RAM
		w_cmd_ack : in std_logic; -- kommt vom RAM
		w_burst_done : out std_logic; -- geht ans RAM
		write_en : in std_logic; -- kommt vom CONTROL
		write_busy : out std_logic -- geht ans CONTROL
	);
	
end DDR2_Write_VHDL;

architecture Verhalten of DDR2_Write_VHDL is

	--------------------------------------------
	-- Interne Signale
	--------------------------------------------
	
	constant clk_anz : integer := 2; -- warte 3 Clockzyklen (2 bis 0 = 3)	
	signal v_counter :  natural range 0 to clk_anz := clk_anz;
	
	type STATE_W_TYPE is (
		W1_NOP,
		W2_WRITE_CMD,
		W3_WAIT_4_ACK1,
		W4_WAIT_3_CLK,
		W5_BURST_HI,
		W6_BURST_OK,
		W7_WAIT_4_ACK0
	);
	signal STATE_W : STATE_W_TYPE := W1_NOP;		

begin

	-----------------------------------------
	-- State-Machine zum
	-- schreiben eines wortes ins RAM
	-----------------------------------------	
	P_Write : process(clk_in,reset_in)
	begin
		if reset_in = '1' then
			-- reset button ist gedrueckt
			STATE_W <= W1_NOP;
			w_command_register <= "000"; -- NOP
			w_burst_done <= '0';
		elsif falling_edge(clk_in) then
			case STATE_W is
				when W1_NOP =>		
					-- warte auf write enable signal
					w_command_register <= "000"; -- NOP
					v_counter <= clk_anz;
					w_burst_done <= '0';
					if write_en = '1' then
						STATE_W <= W2_WRITE_CMD;						
					end if;								
				when W2_WRITE_CMD =>
					-- CMD anlegen
					w_command_register <= "100"; -- WRITE-CMD
					STATE_W <= W3_WAIT_4_ACK1;										
				when W3_WAIT_4_ACK1 =>
					-- warten auf ACK=1 vom RAM
					if w_cmd_ack = '1' then
						STATE_W <= W4_WAIT_3_CLK;
					end if;					
				when W4_WAIT_3_CLK =>
					-- warte 3 Clockzyklen
					if v_counter = 0 then
						-- burst_done auf HI
						w_burst_done <= '1';
						STATE_W <= W5_BURST_HI;						
					else
						w_burst_done <= '0';
						v_counter <= v_counter - 1;
					end if;	
				when W5_BURST_HI =>
					-- NOP anlegen
					w_command_register <= "000"; -- NOP
					STATE_W <= W6_BURST_OK;
				when W6_BURST_OK =>
					-- burst_done auf Lo
					w_burst_done <= '0';
					STATE_W <= W7_WAIT_4_ACK0;
				when W7_WAIT_4_ACK0 =>
					-- warten auf ACK=0 vom RAM
					if w_cmd_ack = '0' then
						STATE_W <= W1_NOP;
					end if;					
				when others =>
					NULL;
			end case;
		end if;
	end process P_Write;
	
	-----------------------------------------
	-- Write-Busy erzeugen
	-----------------------------------------		
	write_busy <= '0' when STATE_W=W1_NOP else '1';	

end Verhalten;

